library neohub;

import 'dart:io';
import 'dart:convert';
import 'dart:async';
import 'dart:typed_data';

/// Abstract Sender type made abstract for testing.
abstract class Sender {
  Future<Map> send(Map message);
}

/// Send commands by logging them for testing
class TestingSender extends Sender {
  final List<Map> log = [];
  final Map response;

  TestingSender(this.response);

  @override
  Future<Map> send(Map message) {
    log.add(message);
    return Future.value(response);
  }
}

/// Send commands to a live server
class SocketSender extends Sender {
  final host;
  final port = 4242;
  final _terminator = [0, 13];

  Socket _sock;

  SocketSender(this.host);

  /// Send the given command [message].
  @override
  Future<Map> send(Map message) async {
    _sock = await Socket.connect(host, port);
    _sock.add(utf8.encode(jsonEncode(message)));
    _sock.add(_terminator);
    final ds = List<int>();
    await for (Uint8List event in _sock) {
      ds.addAll(event);
      if (event[event.length - 1] == 0) {
        break;
      }
    }
    // Don't forget to remove the stupid terminator
    ds.remove(0);
    final resp = jsonDecode(utf8.decode(ds));
    await _sock.close();
    return resp;
  }
}

/// The NeoHub API
class NeoHub {
  final Sender _sender;

  NeoHub.forSender(this._sender);

  NeoHub(String host) : _sender = SocketSender(host);

  /// GET_LIVE_DATA command.
  Future<Map> getLiveData() async {
    final msg = {'GET_LIVE_DATA': 0};
    return _sender.send(msg);
  }

  /// GET_ZONES command.
  Future<Map> getZones() async {
    final msg = {'GET_ZONES': 0};
    return _sender.send(msg);
  }
}
